<?php
namespace Modules\Cadastros\Controllers;

use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Util\SupUtilities;

class MarcaController extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $dataSourceName = "marca";
    protected $connection;

	 public function __construct($crudService, ParameterBag $parameterBag, $connection, SupUtilities $supUtilities) 
	 {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->con = $connection;
        $this->supUtilities = $supUtilities;
    }
    
    public function save(DTO\Request $request, DTO\Response $response) {
        foreach($request->getDataSet()->getRows() as $row){
            if ($row["CDMARCA"] == "") {
                $row["CDMARCA"] = $this->getNextID();
            }
        }
		try {
			$dataSourceName = $this->dataSourceName;
			$rows = $this->crudService->getRowsToSave($request, $dataSourceName);
			$this->crudService->save($dataSourceName, $rows);
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
	}

	public function checkParent(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        $params = array(
    	    "CDMARCA" => $row['CDMARCA']
    	);
    	$sql = "SELECT MARCPROD.NRORG,MARCPROD.CDPRODUTO, MARCPROD.CDMARCA, MARCPROD.VRARREDEMPMAR, PRODUTO.CDARVPROD, PRODUTO.CDNVPRODUTO, PRODUTO.IDPRODATIVPR, PRODUTO.NMPRODUTO FROM MARCPROD, PRODUTO WHERE  (MARCPROD.CDPRODUTO = PRODUTO.CDPRODUTO) AND (MARCPROD.CDMARCA = :CDMARCA) ORDER BY PRODUTO.CDPRODUTO";
    	$stmt = $this->con->prepare($sql);
    	$stmt->execute($params);
		$result = $stmt->fetchAll();
		
    	$isParent = false;
        $cdProdFinSemZ = substr($row['CDPRODFIN'], 0, strpos($row['CDPRODFIN'], "Z"));
		foreach ($result as $produto) {
			
			$posicaoProdPaiInicial = strpos($produto['CDPRODUTO'], $row['CDPRODINI']);
			$posicaoProdFilhoInicial = strpos($row['CDPRODINI'], $produto['CDPRODUTO']);
			
			if(($posicaoProdPaiInicial !== false && $posicaoProdPaiInicial == 0) 
			|| ($posicaoProdFilhoInicial !== false && $posicaoProdFilhoInicial == 0)){
				$isParent = true;
			}
			
			$posicaoProdPaiFinal = strpos($produto['CDPRODUTO'], $row['CDPRODFIN']);
			$posicaoProdFilhoFinal = strpos($row['CDPRODFIN'], $produto['CDPRODUTO']);
			
			if(($posicaoProdPaiFinal !== false && $posicaoProdPaiFinal == 0) 
			|| ($posicaoProdFilhoFinal !== false && $posicaoProdFilhoFinal == 0)){
				$isParent = true;
			}
		}
		$value = array($isParent);
		$response->addDataSet(new DataSet("value", $value));
	}
    
    private function getNextID(){
        /*  copiado e colado do DELPHI, para máxima compatibilidade */
        $query = "SELECT MIN((SUBSTR('00000' || TO_NUMBER(CDMARCA + 1), -5, 5))) AS CDMARCA
                    FROM (SELECT CDMARCA FROM MARCA
                          WHERE CDMARCA <> '99999'
                          UNION ALL
                          SELECT '00000' FROM PARAMGERAL)MARCA
                    WHERE NOT EXISTS (SELECT M.CDMARCA
                                      FROM MARCA M
                                      WHERE SUBSTR('00000' || TO_NUMBER(MARCA.CDMARCA + 1), -5, 5) = M.CDMARCA)";
        $stmt = $this->con->prepare($query);
        $stmt->execute();
		 
		 return $stmt->fetchAll()[0]['CDMARCA'];    
    }
    
    public function returnId(DTO\Request $request, DTO\Response $response) {
		$value = array($this->getNextID());
		$response->addDataSet(new DataSet("value", $value));
	}
	
	public function insertProdutoMarcaCompletoIncompleto(DTO\Request $request, DTO\Response $response)
	{
        try {
        	$row = $request->getRow();
        	
        	$procedure = new StoredProcedure($this->con, 'INSERE_PRODFORNCMP_MARCFORNCMP');
            
            $procedure->addParam(new Param('P_CDFORNECED', Param::PARAM_INPUT,  NULL,              Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDMARCA',    Param::PARAM_INPUT,  $row['CDMARCA'],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODUTO',  Param::PARAM_INPUT,  $row['CDPRODUTO'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDARVPROD',  Param::PARAM_INPUT,  $row['CDARVPROD'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_ARRED',      Param::PARAM_INPUT,  NULL,              Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFORNORIG', Param::PARAM_INPUT,  NULL,              Param::PARAM_TYPE_STR));
            
            $procedure->execute();
        } catch (Exception $e) {
            throw new \Exception('Erro ao inserir produto!');
        }
	}

    public function deleteProduct(DTO\Request $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
        	$query = "DELETE 
        				FROM MARCPROD
					  WHERE
						CDPRODUTO = :OLD_CDPRODUTO AND
						CDMARCA = :OLD_CDMARCA";
            $params = [
                'OLD_CDPRODUTO' => $row['CDPRODUTO'],
                'OLD_CDMARCA'   => $row['CDMARCA']
            ];
            $this->supUtilities->crud($query, $params);
        } catch (Exception $e) {
            throw new \Exception('Erro ao deletar produto!');
        }
	}
}